/*
Module : NTRAY.CPP
Purpose: implementation for a MFC class to encapsulate Shell_NotifyIcon
Created: PJN / NOTSHELL/1 / 14-05-1997
History: None

Copyright (c) 1997 by PJ Naughter.  
All rights reserved.

*/

/////////////////////////////////  Includes  //////////////////////////////////
#include "stdafx.h"
#include "resource.h"
#include "ntray.h"
#include "MenuXP.h"
#include "NeMessengerDlg.h"
#include "main_frame.hpp"

/////////////////////////////////  Macros /////////////////////////////////////

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

const TCHAR UPDATE_GUID[] = _T("EE77F315-BA0A-4ec2-A88F-3A0D6121B831");

#define MAX_TRAY_REPEAT_COUNT 15
IMPLEMENT_DYNAMIC(CTrayNotifyIcon, CObject)



///////////////////////////////// Implementation //////////////////////////////

CTrayNotifyIcon::CTrayNotifyIcon()
{
	memset(&m_NotifyIconData, 0, sizeof(m_NotifyIconData));

	m_bCreated = FALSE;
	m_pWnd = NULL;
	bSetIconFailed = FALSE;
	m_bLogin = TRUE;
}


CTrayNotifyIcon::~CTrayNotifyIcon()
{
	RemoveIcon();
}


void CTrayNotifyIcon::RemoveIcon()
{
	//GT_FLOG(L"Tray m_bCreated is %s(RemoveIcon - 1)", m_bCreated ? L"TRUE" : L"FALSE");

	if (m_bCreated)
	{
		m_NotifyIconData.uFlags = 0;
		Shell_NotifyIcon(NIM_DELETE, &m_NotifyIconData);
		m_bCreated = FALSE;
	}

	//GT_FLOG(L"Tray m_bCreated is %s(:RemoveIcon - 2)", m_bCreated ? L"TRUE" : L"FALSE");
}

BOOL CTrayNotifyIcon::AddToTray()
{
	CSingleLock iconupdateLock(&m_csTrayIcon);
	iconupdateLock.Lock();

	//BOOL bResult = Shell_NotifyIcon(NIM_ADD, &m_NotifyIconData);
	NOTIFYICONDATA modifyicondata;
	memset(&modifyicondata, 0, sizeof(modifyicondata));

	memcpy( &modifyicondata, &m_NotifyIconData, sizeof(NOTIFYICONDATA));
	modifyicondata.uFlags = NIF_ICON | NIF_MESSAGE | NIF_TIP;

	BOOL bResult = Shell_NotifyIcon(NIM_ADD, &modifyicondata);

	//GT_FLOG(L"Tray m_bCreated is %s(AddToTray)", m_bCreated ? L"TRUE" : L"FALSE");

	iconupdateLock.Unlock();

	return  bResult;
}

BOOL CTrayNotifyIcon::Create(CWnd* pNotifyWnd, UINT uID, LPCTSTR pszTooltipText, HICON hIcon, UINT nNotifyMessage)
{
  if (pNotifyWnd == NULL)
    return FALSE;

  int rVal, repeat_count;
  DWORD ErrorCode;

  //Make sure Notification window is valid
  ASSERT(pNotifyWnd && ::IsWindow(pNotifyWnd->GetSafeHwnd()));

  m_pWnd = pNotifyWnd;

  //Make sure we avoid conflict with other messages
  ASSERT(nNotifyMessage >= WM_USER);

  //Tray only supports tooltip text up to 64 characters
  ASSERT(_tcslen(pszTooltipText) <= 64);

  m_NotifyIconData.cbSize = sizeof(m_NotifyIconData);
  m_NotifyIconData.hWnd = pNotifyWnd->GetSafeHwnd();
  m_NotifyIconData.uID = uID;
  m_NotifyIconData.uFlags = NIF_ICON | NIF_MESSAGE | NIF_TIP;
  m_NotifyIconData.uCallbackMessage = nNotifyMessage;
  m_NotifyIconData.hIcon = hIcon;
  _tcscpy_s(m_NotifyIconData.szTip, 128, pszTooltipText);

  repeat_count = 0;
  rVal = AddToTray();
  ErrorCode = GetLastError();
  //GT_FLOG(L"ADDTOTRAY ret=%d:%d", rVal, ErrorCode);

  if ((rVal == 0) && (ErrorCode == ERROR_TIMEOUT)) 
  {
	  do {
		  repeat_count++;
		  Sleep(2000);
		  rVal = Shell_NotifyIcon(NIM_MODIFY, &m_NotifyIconData);
		  //GT_FLOG(L"MODIFYTRAY ret=%d", rVal);
	  } while ((rVal==0) && (repeat_count <= MAX_TRAY_REPEAT_COUNT));
  }

  if (rVal == 0) 
  {
	  DWORD format_retval;
	  LPVOID lpMsgBuf;
	  format_retval = 
		  FormatMessage( 
		  FORMAT_MESSAGE_ALLOCATE_BUFFER | 
		  FORMAT_MESSAGE_FROM_SYSTEM | 
		  FORMAT_MESSAGE_IGNORE_INSERTS,
		  NULL,
		  GetLastError(),
		  0, // Default language
		  (LPTSTR) &lpMsgBuf,
		  0,
		  NULL 
		  );

	  if (format_retval > 0) 
	  {
		  m_NotifyMsg = (LPTSTR)lpMsgBuf;
		  LocalFree( lpMsgBuf );
	  }
	  else 
	  {
		  m_NotifyMsg = "";
	  }
  }

  m_bCreated = rVal;
  //GT_FLOG(L"Tray m_bCreated is %s(Create)", m_bCreated ? L"TRUE" : L"FALSE");
  //GT_FLOG(L"CREATETRAY ret=%d", rVal);

  return rVal;
}


BOOL CTrayNotifyIcon::SetTooltipText(LPCTSTR pszTooltipText)
{
	//GT_FLOG(L"Tray m_bCreated is %s(SetTooltipText)", m_bCreated ? L"TRUE" : L"FALSE");
	if (!m_bCreated)
		return FALSE;

	m_NotifyIconData.uFlags = NIF_TIP;
	_tcscpy_s(m_NotifyIconData.szTip, 128, pszTooltipText);

	return Shell_NotifyIcon(NIM_MODIFY, &m_NotifyIconData);
}


BOOL CTrayNotifyIcon::SetTooltipText(UINT nID)
{
	CString sToolTipText;
	VERIFY(sToolTipText.LoadString(nID));

	return SetTooltipText(sToolTipText);
}


BOOL CTrayNotifyIcon::SetIcon(HICON hIcon)
{
  BOOL ret_val;

  //GT_FLOG(L"Tray m_bCreated is %s(SetIcon)", m_bCreated ? L"TRUE" : L"FALSE");
  //if (!m_bCreated)
  //  return FALSE;

  CSingleLock iconupdateLock(&m_csTrayIcon);
  iconupdateLock.Lock();
  m_NotifyIconData.uFlags = NIF_ICON;
  m_NotifyIconData.hIcon = hIcon;

  ret_val = Shell_NotifyIcon(NIM_MODIFY, &m_NotifyIconData);

  if (ret_val == FALSE) {
	  bSetIconFailed = TRUE;
  }
  else {
	  bSetIconFailed = FALSE;
  }

  iconupdateLock.Unlock();
  return ret_val;
}


BOOL CTrayNotifyIcon::SetIcon(LPCTSTR lpIconName)
{
  HICON hIcon = AfxGetApp()->LoadIcon(lpIconName);

  return SetIcon(hIcon);
}


BOOL CTrayNotifyIcon::SetIcon(UINT nIDResource)
{
  HICON hIcon = AfxGetApp()->LoadIcon(nIDResource);

  return SetIcon(hIcon);
}


BOOL CTrayNotifyIcon::SetStandardIcon(LPCTSTR lpIconName)
{
  HICON hIcon = LoadIcon(NULL, lpIconName);

  return SetIcon(hIcon);
}


BOOL CTrayNotifyIcon::SetStandardIcon(UINT nIDResource)
{
  HICON hIcon = LoadIcon(NULL, MAKEINTRESOURCE(nIDResource));

  return SetIcon(hIcon);
}
 

BOOL CTrayNotifyIcon::SetNotificationWnd(CWnd* pNotifyWnd)
{
	//GT_FLOG(L"Tray m_bCreated is %s(SetNotificationWnd)", m_bCreated ? L"TRUE" : L"FALSE");
  if (pNotifyWnd == NULL || !m_bCreated)
    return FALSE;

  //Make sure Notification window is valid
  ASSERT(pNotifyWnd && ::IsWindow(pNotifyWnd->GetSafeHwnd()));

  m_pWnd = pNotifyWnd;
  m_NotifyIconData.hWnd = pNotifyWnd->GetSafeHwnd();
  m_NotifyIconData.uFlags = 0;

  return Shell_NotifyIcon(NIM_MODIFY, &m_NotifyIconData);
}


CString CTrayNotifyIcon::GetTooltipText() const
{
	//GT_FLOG(L"Tray m_bCreated is %s(GetTooltipText)", m_bCreated ? L"TRUE" : L"FALSE");

	CString sText;
	if (m_bCreated)
		sText = m_NotifyIconData.szTip;

  return sText;
}


HICON CTrayNotifyIcon::GetIcon() const
{
	//GT_FLOG(L"Tray m_bCreated is %s(GetIcon)", m_bCreated ? L"TRUE" : L"FALSE");

  HICON hIcon = NULL;
  if (m_bCreated)
    hIcon = m_NotifyIconData.hIcon;

  return hIcon;
}


CWnd* CTrayNotifyIcon::GetNotificationWnd() const
{
  return m_pWnd;
}


LRESULT CTrayNotifyIcon::OnTrayNotification(WPARAM wID, LPARAM lEvent)
{
  //Return quickly if its not for this tray icon
  if (wID != m_NotifyIconData.uID)
    return 0L;

  // ̾αװ ִ ̹Ƿ ޴  ƾ.
  //if (g_ModalBoxCount != 0)
	  //return 0L;

  //As a default action use a menu resource with the same id 
  //as this was created with
  CMenuXP menu;
  if (!menu.LoadMenu(m_NotifyIconData.uID))
    return 0;

  CMenu* pSubMenu = menu.GetSubMenu(0);
  if (!pSubMenu) 
    return 0;

  if (lEvent == WM_RBUTTONUP)
  {
    //Clicking with right button brings up a context menu

	  CMenuXP* pMenuStatus = NULL;

	  if (m_bLogin)
	  {
		  pMenuStatus = new CMenuXP();
		  MainFrame::MakeAboutMeSatusPopupMenu(pMenuStatus);
		  pSubMenu->InsertMenu(0, MF_POPUP | MF_BYPOSITION, (UINT_PTR)pMenuStatus->m_hMenu, 
			  LoadStringFromResource(IDS_TRAY_CHANGE_MYSTATUS));
	  }
	  else
	  {
		  pSubMenu->DeleteMenu(0, MF_BYPOSITION);
	  }

	  // Make first menu item the default (bold font)
    ::SetMenuDefaultItem(pSubMenu->m_hMenu, ID__MNOPEN, MF_BYCOMMAND);

	HANDLE nsisMutex = ::CreateMutex( NULL, TRUE, UPDATE_GUID );
	if (NULL == nsisMutex)
	{
		pSubMenu->EnableMenuItem(ID__UPDATE, MF_BYCOMMAND | MF_DISABLED | MF_GRAYED);
	}
	else
	{
		ReleaseMutex(nsisMutex);
		CloseHandle(nsisMutex);
		pSubMenu->EnableMenuItem(ID__UPDATE, MF_BYCOMMAND | MF_ENABLED);
	}

	HWND hWndMenu = NULL;

    //Display and track the popup menu
    CPoint pos(0, 0);
    GetCursorPos(&pos);
	if(m_bLogin == TRUE) // login
	{
		pSubMenu->EnableMenuItem(ID__MNLOGIN, MF_BYCOMMAND | MF_DISABLED | MF_GRAYED);
		pSubMenu->EnableMenuItem(ID__MNLOGOUT, MF_BYCOMMAND | MF_ENABLED);
		pSubMenu->EnableMenuItem(ID__TRAYSETTING, MF_BYCOMMAND | MF_ENABLED);
		hWndMenu = GET_MESSENGER_DLG()->GetMainFrame()->GetSafeHwnd();
	}
	else // logout
	{
		pSubMenu->EnableMenuItem(ID__MNLOGIN, MF_BYCOMMAND | MF_ENABLED);
		pSubMenu->EnableMenuItem(ID__MNLOGOUT,MF_BYCOMMAND | MF_DISABLED | MF_GRAYED);
		pSubMenu->EnableMenuItem(ID__TRAYSETTING, MF_BYCOMMAND | MF_DISABLED | MF_GRAYED);
		hWndMenu = GET_MESSENGER_DLG()->GetSafeHwnd();
	}

	SetForegroundWindow(m_NotifyIconData.hWnd);

	int nCmdID = pSubMenu->TrackPopupMenu(TPM_LEFTBUTTON | TPM_RETURNCMD | TPM_RIGHTBUTTON, pos.x, pos.y, 
		CWnd::FromHandle(hWndMenu), NULL);
	if (pMenuStatus)
	{
		delete pMenuStatus;
        pMenuStatus = NULL;
	}
	if ((GTUserStatusType)nCmdID >= ustONLINE && (GTUserStatusType)nCmdID <= ustAUTOAWAY)
	{
		//status command
		::SendMessage(m_NotifyIconData.hWnd, UM_INFO_ABOUTME_STATUS, nCmdID, 0);
	}
	else
	{
		//other command
		::SendMessage(m_NotifyIconData.hWnd, WM_COMMAND, nCmdID, 0);
	}
    //::TrackPopupMenu(pSubMenu->m_hMenu, TPM_LEFTBUTTON, pos.x, pos.y, 0, m_NotifyIconData.hWnd, NULL);

	PostMessage(m_NotifyIconData.hWnd, WM_NULL, 0, 0);


  } 
  else if (lEvent == WM_LBUTTONDBLCLK) 
  {
    ::SendMessage(m_NotifyIconData.hWnd, WM_COMMAND, ID__MNOPEN, 0); // Ŭ ϸ ""
  }

  return 1; // handled
}


void CTrayNotifyIcon::SetLogin(BOOL bLogin)
{
	m_bLogin = bLogin;
}
